@@ -101,15 +101,21 @@ ArvBuffer *	arv_camera_acquisition			(ArvCamera *camera, guint64 timeout);
 void			arv_camera_set_acquisition_mode 	(ArvCamera *camera, ArvAcquisitionMode value);
 ArvAcquisitionMode 	arv_camera_get_acquisition_mode 	(ArvCamera *camera);
 
+void 		arv_camera_set_frame_count	(ArvCamera *camera, gint64 frame_count);
+gint64		arv_camera_get_frame_count	(ArvCamera *camera);
+void		arv_camera_get_frame_count_bounds	(ArvCamera *camera, gint64 *min, gint64 *max);
+
 gboolean 	arv_camera_is_frame_rate_available 	(ArvCamera *camera);
 
 void		arv_camera_set_frame_rate		(ArvCamera *camera, double frame_rate);
 double 		arv_camera_get_frame_rate 		(ArvCamera *camera);
-void		arv_camera_get_frame_rate_bounds 	(ArvCamera * camera, double *min, double *max);
+void		arv_camera_get_frame_rate_bounds 	(ArvCamera *camera, double *min, double *max);
 void		arv_camera_set_trigger			(ArvCamera *camera, const char *source);
 void 		arv_camera_set_trigger_source		(ArvCamera *camera, const char *source);
 const char *	arv_camera_get_trigger_source		(ArvCamera *camera);
-
+const char **	arv_camera_get_available_trigger_sources(ArvCamera *camera, guint *n_sources);
+const char**    arv_camera_get_available_triggers       (ArvCamera *camera, guint *n_triggers);
+void            arv_camera_clear_triggers               (ArvCamera* camera);
 void 		arv_camera_software_trigger 		(ArvCamera *camera);
 
 gboolean 	arv_camera_is_exposure_time_available	(ArvCamera *camera);
@@ -152,6 +158,14 @@ guint		arv_camera_gv_auto_packet_size		(ArvCamera *camera);
 
 void 		arv_camera_gv_set_stream_options 	(ArvCamera *camera, ArvGvStreamOption options);
 
+/* USB3Vision specific API */
+
+gboolean        arv_camera_is_uv_device                 	(ArvCamera *camera);
+gboolean        arv_camera_uv_is_bandwidth_control_available 	(ArvCamera *camera);
+void            arv_camera_uv_set_bandwidth             	(ArvCamera *camera, guint bandwidth);
+guint           arv_camera_uv_get_bandwidth             	(ArvCamera *camera);
+void            arv_camera_uv_get_bandwidth_bounds      	(ArvCamera *camera, guint* min, guint* max);
+
 /* Chunk data */
 
 void 			arv_camera_set_chunk_mode 	(ArvCamera *camera, gboolean is_active);
