@@ -171,11 +171,11 @@ arv_gc_enumeration_set_string_value (ArvGcEnumeration *enumeration, const char *
 			GError *local_error = NULL;
 			gint64 enum_value;
 
-			arv_log_genicam ("[GcEnumeration::set_string_value] value = %d - string = %s",
-					 &enumeration->value, value);
-
 			enum_value = arv_gc_enum_entry_get_value (iter->data, &local_error);
 
+			arv_log_genicam ("[GcEnumeration::set_string_value] value = %d - string = %s",
+					 enum_value, value);
+
 			if (local_error != NULL) {
 				g_propagate_error (error, local_error);
 				return;
@@ -201,7 +201,7 @@ arv_gc_enumeration_get_int_value (ArvGcEnumeration *enumeration, GError **error)
 	g_return_val_if_fail (ARV_IS_GC_ENUMERATION (enumeration), 0);
 	g_return_val_if_fail (error == NULL || *error == NULL, 0);
 
-	if (enumeration->value == NULL) 
+	if (enumeration->value == NULL)
 		return 0;
 
 	value = arv_gc_property_node_get_int64 (enumeration->value, &local_error);
