@@ -36,13 +36,40 @@
 #include <arvenumtypes.h>
 #include <string.h>
 #include <stdlib.h>
+#ifndef __APPLE__
 #include <linux/ip.h>
+#endif
 #include <netinet/udp.h>
 
 static GObjectClass *parent_class = NULL;
 
 /* Shared data (main thread - heartbeat) */
 
+#ifdef __APPLE__
+struct iphdr
+  {
+#if __BYTE_ORDER == __LITTLE_ENDIAN
+    unsigned int ihl:4;
+    unsigned int version:4;
+#elif __BYTE_ORDER == __BIG_ENDIAN
+    unsigned int version:4;
+    unsigned int ihl:4;
+#else
+# error  "Please fix <bits/endian.h>"
+#endif
+    u_int8_t tos;
+    u_int16_t tot_len;
+    u_int16_t id;
+    u_int16_t frag_off;
+    u_int8_t ttl;
+    u_int8_t protocol;
+    u_int16_t check;
+    u_int32_t saddr;
+    u_int32_t daddr;
+    /*The options start here. */
+  };
+#endif
+
 typedef struct {
 #if GLIB_CHECK_VERSION(2,32,0)
 	GMutex mutex;
@@ -96,7 +123,7 @@ static GRegex *arv_gv_device_url_regex = NULL;
 }
 
 static gboolean
-_read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *buffer, GError **error)
+_read_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvcpPacket *packet;
 	size_t packet_size;
@@ -177,7 +204,7 @@ _read_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *b
 }
 
 static gboolean
-_write_memory (ArvGvDeviceIOData *io_data, guint32 address, guint32 size, void *buffer, GError **error)
+_write_memory (ArvGvDeviceIOData *io_data, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvcpPacket *packet;
 	size_t packet_size;
@@ -628,7 +655,7 @@ arv_gv_device_auto_packet_size (ArvGvDevice *gv_device)
 					read_count = g_socket_receive (socket, buffer, 8192, NULL, NULL);
 				else
 					read_count = 0;
-			/* Discard late packets, read_count should be equal to packet size minus IP and UDP headers */
+				/* Discard late packets, read_count should be equal to packet size minus IP and UDP headers */
 			} while (n_events != 0 && read_count != (current_size - sizeof (struct iphdr) - sizeof (struct udphdr)));
 
 			n_tries++;
@@ -1039,7 +1066,7 @@ arv_gv_device_get_genicam (ArvDevice *device)
 }
 
 static gboolean
-arv_gv_device_read_memory (ArvDevice *device, guint32 address, guint32 size, void *buffer, GError **error)
+arv_gv_device_read_memory (ArvDevice *device, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvDevice *gv_device = ARV_GV_DEVICE (device);
 	int i;
@@ -1057,7 +1084,7 @@ arv_gv_device_read_memory (ArvDevice *device, guint32 address, guint32 size, voi
 }
 
 static gboolean
-arv_gv_device_write_memory (ArvDevice *device, guint32 address, guint32 size, void *buffer, GError **error)
+arv_gv_device_write_memory (ArvDevice *device, guint64 address, guint32 size, void *buffer, GError **error)
 {
 	ArvGvDevice *gv_device = ARV_GV_DEVICE (device);
 	int i;
@@ -1075,7 +1102,7 @@ arv_gv_device_write_memory (ArvDevice *device, guint32 address, guint32 size, vo
 }
 
 static gboolean
-arv_gv_device_read_register (ArvDevice *device, guint32 address, guint32 *value, GError **error)
+arv_gv_device_read_register (ArvDevice *device, guint64 address, guint32 *value, GError **error)
 {
 	ArvGvDevice *gv_device = ARV_GV_DEVICE (device);
 
@@ -1083,7 +1110,7 @@ arv_gv_device_read_register (ArvDevice *device, guint32 address, guint32 *value,
 }
 
 static gboolean
-arv_gv_device_write_register (ArvDevice *device, guint32 address, guint32 value, GError **error)
+arv_gv_device_write_register (ArvDevice *device, guint64 address, guint32 value, GError **error)
 {
 	ArvGvDevice *gv_device = ARV_GV_DEVICE (device);
 
@@ -1267,7 +1294,7 @@ arv_gv_device_finalize (GObject *object)
  * @device: a #ArvGvDevice
  *
  * Returns: (transfer none): the device host interface IP address.
- * 
+ *
  * Since: 0.2.0
  */
 
