@@ -127,7 +127,7 @@ arv_update_device_list (void)
 
 	for (i = 0; i < G_N_ELEMENTS (interfaces); i++) {
 		ArvInterface *interface;
-		
+
 		if (interfaces[i].is_available) {
 			interface = interfaces[i].get_interface_instance ();
 			arv_interface_update_device_list (interface);
@@ -201,6 +201,23 @@ arv_get_device_serial_nbr (unsigned int index)
 	return arv_get_info (index, arv_interface_get_device_serial_nbr);
 }
 
+/**
+ * arv_get_device_address:
+ * @index: device index
+ *
+ * The index of a device may change after a call to arv_update_device_list().
+ *
+ * Returns: The address of the device corresponding to @index as a string.
+ *
+ * Since: 0.6.0
+ */
+
+const char *
+arv_get_device_address (unsigned int index)
+{
+	return arv_get_info (index, arv_interface_get_device_address);
+}
+
 /**
  * arv_open_device:
  * @device_id: (allow-none): a device identifier string
