@@ -137,7 +137,7 @@ typedef struct ARAVIS_PACKED_STRUCTURE {
 
 typedef struct ARAVIS_PACKED_STRUCTURE {
 	guint64 address;
-	guint16 unknown;
+        guint16 unknown;	/* Listed as reserved, always 0 */
 	guint16 size;
 } ArvUvcpReadMemoryCmdInfos;
 
@@ -189,7 +189,10 @@ typedef struct ARAVIS_PACKED_STRUCTURE {
 } ArvUvcpPacket;
 
 typedef struct ARAVIS_PACKED_STRUCTURE {
-	guint64 unknown0;
+	guint16 file_version_subminor;
+	guint8 file_version_minor;
+	guint8 file_version_major;
+	guint32 schema;
 	guint64 address;
 	guint64 size;
 	guint64 unknown3;
@@ -199,12 +202,35 @@ typedef struct ARAVIS_PACKED_STRUCTURE {
 	guint64 unknown7;
 } ArvUvcpManifestEntry;
 
+/**
+ * ArvUvcpManifestSchemaType:
+ * @ARV_UVCP_SCHEMA_RAW: uncompressed genicam data
+ * @ARV_UVCP_SCHEMA_ZIP: zipped genicam data
+ *
+ * This is packed into the 32-bit schema type as bits 10-15
+ */
+
+typedef enum
+{
+	ARV_UVCP_SCHEMA_RAW = 0x0,
+	ARV_UVCP_SCHEMA_ZIP = 0x1
+}
+ArvUvcpManifestSchemaType;
+
+static inline ArvUvcpManifestSchemaType
+arv_uvcp_manifest_entry_get_schema_type (ArvUvcpManifestEntry *entry)
+{
+	g_return_val_if_fail (entry != NULL, ARV_UVCP_SCHEMA_RAW);
+
+	return (entry->schema >> 10) & 0x0000001f;
+}
+
 #undef ARAVIS_PACKED_STRUCTURE
 
 void 			arv_uvcp_packet_free 			(ArvUvcpPacket *packet);
-ArvUvcpPacket * 	arv_uvcp_packet_new_read_memory_cmd 	(guint32 address, guint32 size,
+ArvUvcpPacket * 	arv_uvcp_packet_new_read_memory_cmd 	(guint64 address, guint32 size,
 								 guint16 packet_id, size_t *packet_size);
-ArvUvcpPacket * 	arv_uvcp_packet_new_write_memory_cmd	(guint32 address, guint32 size,
+ArvUvcpPacket * 	arv_uvcp_packet_new_write_memory_cmd	(guint64 address, guint32 size,
 								 guint16 packet_id, size_t *packet_size);
 char * 			arv_uvcp_packet_to_string 		(const ArvUvcpPacket *packet);
 void 			arv_uvcp_packet_debug 			(const ArvUvcpPacket *packet, ArvDebugLevel level);
